
# q-expansion of newform 648.2.d.i, downloaded from the LMFDB on 06 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 648
weight = 2
poly_data = [1, 0, -1, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 1, 1, 0], 1], [[0, 0, 0, 1], 1], [[0, 1, -1, 0], 1]]

hecke_ring_character_values = [[487, [1, 0, 0, 0]], [325, [-1, 0, 0, 0]], [569, [1, 0, 0, 0]]]
aps_data = [[0, 1, -1, 0], [0, 0, 0, 0], [1, -1, -2, 1], [1, -1, 1, -1], [1, -1, -3, 1], [2, -2, 1, 2], [-2, -1, 1, -1], [1, -1, 5, 1], [7, 1, -1, 1], [-3, 3, 0, -3], [2, 0, 0, 0], [0, 0, -5, 0], [8, -2, 2, -2], [-5, 5, 1, -5], [0, 2, -2, 2], [-4, 4, -6, -4], [6, -6, 2, 6], [0, 0, 9, 0], [-1, 1, 3, -1], [-3, 5, -5, 5], [9, 0, 0, 0], [7, -1, 1, -1], [0, 0, 14, 0], [-10, -1, 1, -1], [-8, 0, 0, 0], [0, 0, -8, 0], [0, -2, 2, -2], [6, -6, 0, 6], [-8, 8, -1, -8], [-6, -1, 1, -1], [-5, -5, 5, -5], [3, -3, 5, 3], [-12, -3, 3, -3], [2, -2, -16, 2], [-3, 3, 0, -3], [0, 8, -8, 8], [2, -2, 1, 2], [2, -2, 10, 2], [-13, -1, 1, -1], [1, -1, 0, 1], [0, 0, 14, 0], [-2, 2, -10, -2], [1, 9, -9, 9], [-3, 8, -8, 8], [7, -7, 8, 7], [-8, 6, -6, 6], [-11, 11, 7, -11], [11, 7, -7, 7], [1, -1, -5, 1], [6, -6, -1, 6], [2, 1, -1, 1], [-6, 8, -8, 8], [-1, -2, 2, -2], [-1, 1, -9, -1], [-12, 5, -5, 5], [8, -14, 14, -14], [1, -1, -24, 1], [9, 1, -1, 1], [-8, 8, -4, -8], [12, 1, -1, 1], [6, -6, 4, 6], [-5, 5, -12, -5], [-12, 12, -6, -12], [-2, -10, 10, -10], [-1, 2, -2, 2], [3, -3, 4, 3], [-7, 7, 1, -7], [4, 0, 0, 0], [1, -1, -5, 1], [-8, 8, -2, -8], [0, -4, 4, -4], [-11, 7, -7, 7], [6, -2, 2, -2], [4, -4, -16, 4], [0, 0, -10, 0], [-15, -3, 3, -3], [4, -4, -10, 4], [-2, 2, 29, -2], [6, -9, 9, -9], [1, 16, -16, 16], [2, -2, -18, 2], [0, 0, -13, 0], [6, -16, 16, -16], [-1, -12, 12, -12], [-30, 0, 0, 0], [-10, 10, 12, -10], [-18, -4, 4, -4], [11, -10, 10, -10], [10, -10, -24, 10], [10, 2, -2, 2], [-12, 12, 4, -12], [-7, 5, -5, 5], [2, -16, 16, -16], [-3, 3, -19, -3], [-3, 3, 3, -3], [-18, 10, -10, 10], [-10, 10, 12, -10], [6, 12, -12, 12], [-1, 1, -15, -1], [18, -18, 7, 18], [0, 0, 26, 0], [11, -11, 6, 11], [14, -14, 0, 14], [-18, -9, 9, -9], [-14, 14, 2, -14], [-21, -14, 14, -14], [-25, 25, -1, -25], [-30, 9, -9, 9], [16, -14, 14, -14], [-7, 14, -14, 14], [7, -19, 19, -19], [-6, 6, 14, -6], [-14, -7, 7, -7], [14, -14, 18, 14], [-20, 12, -12, 12], [4, 11, -11, 11], [6, -6, 4, 6], [3, 27, -27, 27], [24, -24, 6, 24], [11, -11, 9, 11], [-18, 18, 1, -18], [-13, -14, 14, -14], [-10, 10, 12, -10], [-22, 22, -4, -22], [-1, 1, 11, -1], [-7, 7, -2, -7], [6, -6, -37, 6], [-29, -11, 11, -11], [-27, -13, 13, -13], [2, -2, -26, 2], [-6, 6, -20, -6], [-10, -12, 12, -12], [5, -13, 13, -13], [14, -14, 6, 14], [10, 15, -15, 15], [-29, -6, 6, -6], [5, -5, -32, 5], [17, -17, 15, 17], [1, -1, 14, 1], [-30, -11, 11, -11], [-12, 12, 28, -12], [-9, 9, -14, -9], [-28, 6, -6, 6], [5, -5, 21, 5], [18, -18, -22, 18], [47, -3, 3, -3], [4, -4, 12, 4], [-12, 7, -7, 7], [0, 0, -2, 0], [-3, -13, 13, -13], [22, -22, -19, 22], [14, 8, -8, 8], [10, -10, 16, 10], [-3, 3, -3, 3], [6, -6, -22, 6], [15, 1, -1, 1], [-9, 15, -15, 15], [22, -13, 13, -13], [17, -14, 14, -14], [-21, 21, -18, -21], [-3, 3, 15, -3], [18, 5, -5, 5], [5, 3, -3, 3], [-7, 7, 39, -7], [-2, -12, 12, -12], [2, -2, 2, -2], [-13, 9, -9, 9], [-4, 4, 21, -4]]
